@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-# COPYRIGHT (C) 2020-2022 Nicotine+ Contributors
+# COPYRIGHT (C) 2020-2023 Nicotine+ Contributors
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -20,22 +20,19 @@
 import subprocess
 import sys
 
-""" Script used to create new Nicotine+ releases on PyPi """
-
 
 def create_packages():
-    """ Prepare source distribution and wheel """
+    """Prepare source distribution and wheel."""
 
-    for target in ("sdist", "bdist_wheel"):
-        subprocess.check_call([sys.executable, "setup.py", target])
+    subprocess.check_call([sys.executable, "-m", "build", "--sdist", "--wheel"])
 
 
 def upload_packages():
-    """ Upload release to PyPI """
+    """Upload release to PyPI."""
 
     subprocess.check_call([sys.executable, "-m", "twine", "upload", "dist/*"])
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     create_packages()
     upload_packages()
