@@ -25,14 +25,13 @@ def install_pacman():
     """Install dependencies from the main MinGW repos."""
 
     arch = os.environ.get("ARCH", "x86_64")
-    prefix = "mingw-w64-clang-"
 
     if arch == "arm64":
-        prefix += "aarch64"
+        prefix = "mingw-w64-clang-aarch64"
         mingw_type = "clangarm64"
     else:
-        prefix += "x86_64"
-        mingw_type = "clang64"
+        prefix = "mingw-w64-x86_64"
+        mingw_type = "mingw64"
 
     packages = [f"{prefix}-ca-certificates",
                 f"{prefix}-gettext-tools",
@@ -40,16 +39,18 @@ def install_pacman():
                 f"{prefix}-libadwaita",
                 f"{prefix}-python-build",
                 f"{prefix}-python-cx-freeze",
+                f"{prefix}-python-gobject",
                 f"{prefix}-python-pycodestyle",
                 f"{prefix}-python-pylint",
-                f"{prefix}-python-gobject",
+                f"{prefix}-python-setuptools",
+                f"{prefix}-python-wheel",
                 f"{prefix}-webp-pixbuf-loader"]
 
     subprocess.check_call(["pacman", "--noconfirm", "-S", "--needed"] + packages)
 
-    # Downgrade libadwaita for now due to dark mode not being applied on startup
-    # https://gitlab.gnome.org/GNOME/libadwaita/-/issues/932
-    downgrade_packages = [f"{prefix}-libadwaita-1.5.3-1-any.pkg.tar.zst"]
+    # Downgrade GTK due to crash on certain systems
+    # https://github.com/nicotine-plus/nicotine-plus/issues/3210
+    downgrade_packages = [f"{prefix}-gtk4-4.16.2-1-any.pkg.tar.zst"]
 
     for package in downgrade_packages:
         subprocess.check_call(["curl", "-O", f"https://repo.msys2.org/mingw/{mingw_type}/{package}"])
