@@ -56,41 +56,30 @@ class RoomWall(Popover):
         newline = "\n"
         messages = [f"> [{user}] {msg.replace(newline, ' ')}" for user, msg in reversed(list(tickers.items()))]
 
-        self.message_view.append_line("\n".join(messages))
+        self.message_view.append_line(newline.join(messages))
         self.message_view.place_cursor_at_line(0)
 
-    def _clear_room_wall_message(self, update_list=True):
+    def on_set_room_wall_message(self, *_args):
 
         entry_text = self.message_entry.get_text()
-        self.message_entry.set_text("")
+        core.chatrooms.request_update_ticker(self.room, entry_text)
 
         core.chatrooms.joined_rooms[self.room].tickers.pop(core.users.login_username, None)
         self.message_view.clear()
 
-        if update_list:
-            core.chatrooms.request_update_ticker(self.room, "")
-            self._update_message_list()
-
-        return entry_text
-
-    def on_set_room_wall_message(self, *_args):
-
-        entry_text = self._clear_room_wall_message(update_list=False)
-        core.chatrooms.request_update_ticker(self.room, entry_text)
-
         if entry_text:
-            user = core.users.login_username
-            self.message_view.append_line(f"> [{user}] {entry_text}")
+            self.message_view.append_line(f"> [{core.users.login_username}] {entry_text}")
+            self.message_entry.set_text("")
 
         self._update_message_list()
 
     def on_icon_pressed(self, _entry, icon_pos, *_args):
 
-        if icon_pos == Gtk.EntryIconPosition.PRIMARY:
-            self.on_set_room_wall_message()
-            return
+        if icon_pos == Gtk.EntryIconPosition.SECONDARY:
+            # Clear message
+            self.message_entry.set_text("")
 
-        self._clear_room_wall_message()
+        self.on_set_room_wall_message()
 
     def _on_show(self, *_args):
 
@@ -98,8 +87,7 @@ class RoomWall(Popover):
         self._update_message_list()
 
         login_username = core.users.login_username
-        message = core.chatrooms.joined_rooms[self.room].tickers.get(login_username)
+        message = core.chatrooms.joined_rooms[self.room].tickers.get(login_username, "")
 
-        if message:
-            self.message_entry.set_text(message)
-            self.message_entry.select_region(0, -1)
+        self.message_entry.set_text(message)
+        self.message_entry.select_region(0, -1)
