@@ -28,7 +28,6 @@ from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import Gtk
 
-from pynicotine import slskmessages
 from pynicotine.config import config
 from pynicotine.core import core
 from pynicotine.gtkgui.application import GTK_API_VERSION
@@ -38,6 +37,7 @@ from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
 from pynicotine.gtkgui.widgets.theme import USER_STATUS_ICON_NAMES
 from pynicotine.gtkgui.widgets.theme import add_css_class
 from pynicotine.gtkgui.widgets.theme import remove_css_class
+from pynicotine.slskmessages import UserStatus
 
 
 class TabLabel:
@@ -284,6 +284,7 @@ class IconNotebook:
         self.widget = Gtk.Notebook(enable_popup=False, scrollable=True, show_border=False, visible=True)
 
         self.pages_button_container = Gtk.Box(halign=Gtk.Align.CENTER, visible=(self.parent_page is not None))
+        self.pages_button = Gtk.MenuButton(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, visible=True)
         self.widget.set_action_widget(self.pages_button_container, Gtk.PackType.END)
 
         if parent_page is not None:
@@ -293,9 +294,7 @@ class IconNotebook:
         if GTK_API_VERSION >= 4:
             parent.append(self.widget)
 
-            self.pages_button = Gtk.MenuButton(
-                has_frame=False, halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, visible=True
-            )
+            self.pages_button.set_has_frame(False)                                 # pylint: disable=no-member
             self.pages_button.set_create_popup_func(self.on_pages_button_pressed)  # pylint: disable=no-member
             self.pages_button_container.append(self.pages_button)                  # pylint: disable=no-member
 
@@ -323,8 +322,8 @@ class IconNotebook:
         else:
             parent.add(self.widget)
 
-            self.pages_button = Gtk.Button(halign=Gtk.Align.CENTER, valign=Gtk.Align.CENTER, visible=True)
-            self.pages_button.connect("clicked", self.on_pages_button_pressed)
+            self.pages_button.set_use_popover(False)            # pylint: disable=no-member
+            self.pages_button.connect("toggled", self.on_pages_button_pressed)
             self.pages_button_container.add(self.pages_button)  # pylint: disable=no-member
 
             self.widget.add_events(  # pylint: disable=no-member
@@ -337,11 +336,10 @@ class IconNotebook:
         Accelerator("Left", self.widget, self.on_arrow_accelerator)
         Accelerator("Right", self.widget, self.on_arrow_accelerator)
 
-        self.popup_menu_pages = PopupMenu(self.window.application, self.pages_button, connect_events=False)
+        self.popup_menu_pages = PopupMenu(self.window.application)
         self.update_pages_menu_button()
 
-        if GTK_API_VERSION >= 4:
-            self.pages_button.set_menu_model(self.popup_menu_pages.model)
+        self.pages_button.set_menu_model(self.popup_menu_pages.model)
 
     def destroy(self):
 
@@ -388,14 +386,14 @@ class IconNotebook:
             tab_label = self.get_tab_label(page)
             tab_label.set_close_button_visibility(config.sections["ui"]["tabclosers"])
 
-    def append_page(self, page, text, focus_callback=None, close_callback=None, full_text=None, user=None):
-        self.insert_page(page, text, focus_callback, close_callback, full_text, user, position=-1)
+    def append_page(self, page, text, focus_callback=None, close_callback=None, user=None):
+        self.insert_page(page, text, focus_callback, close_callback, user, position=-1)
 
-    def insert_page(self, page, text, focus_callback=None, close_callback=None, full_text=None, user=None,
+    def insert_page(self, page, text, focus_callback=None, close_callback=None, user=None,
                     position=None):
 
-        if full_text is None:
-            full_text = text
+        full_text = text
+        text = (text[:25] + "…") if len(text) > 25 else text
 
         self.tab_labels[page] = tab_label = TabLabel(
             text, full_text, close_button_visible=config.sections["ui"]["tabclosers"], close_callback=close_callback)
@@ -416,11 +414,11 @@ class IconNotebook:
         self.parent.set_visible(True)
 
         if user is not None:
-            status = core.users.statuses.get(user, slskmessages.UserStatus.OFFLINE)
+            status = core.users.statuses.get(user, UserStatus.OFFLINE)
             self.set_user_status(page, text, status)
 
-    def prepend_page(self, page, text, focus_callback=None, close_callback=None, full_text=None, user=None):
-        self.insert_page(page, text, focus_callback, close_callback, full_text, user, position=0)
+    def prepend_page(self, page, text, focus_callback=None, close_callback=None, user=None):
+        self.insert_page(page, text, focus_callback, close_callback, user, position=0)
 
     def restore_removed_page(self, *_args):
         if self.recently_removed_pages:
@@ -606,10 +604,10 @@ class IconNotebook:
 
     def set_user_status(self, page, user, status):
 
-        if status == slskmessages.UserStatus.AWAY:
+        if status == UserStatus.AWAY:
             status_text = _("Away")
 
-        elif status == slskmessages.UserStatus.ONLINE:
+        elif status == UserStatus.ONLINE:
             status_text = _("Online")
 
         else:
@@ -702,13 +700,10 @@ class IconNotebook:
     def on_show_parent_page(self, *_args):
         self.emit_switch_page_signal()
 
-    def on_pages_button_pressed(self, *args):
+    def on_pages_button_pressed(self, *_args):
 
-        if GTK_API_VERSION >= 4:
-            popover, *_unused = args
-
-            if not popover.is_visible():
-                return
+        if GTK_API_VERSION == 3 and not self.pages_button.get_active():
+            return
 
         self.popup_menu_pages.clear()
 
@@ -744,9 +739,6 @@ class IconNotebook:
         self.popup_menu_pages.update_model()
         self.popup_menu_pages.actions[_("Re_open Closed Tab")].set_enabled(bool(self.recently_removed_pages))
 
-        if GTK_API_VERSION == 3:
-            self.popup_menu_pages.popup(pos_x=0, pos_y=0)
-
     def on_tab_scroll_event(self, _widget, event):
 
         current_page = self.get_current_page()
