@@ -75,6 +75,7 @@ from pynicotine.slskmessages import Relogged
 from pynicotine.slskmessages import ResetDistributed
 from pynicotine.slskmessages import ServerConnect
 from pynicotine.slskmessages import ServerDisconnect
+from pynicotine.slskmessages import ServerReconnect
 from pynicotine.slskmessages import SetDownloadLimit
 from pynicotine.slskmessages import SetUploadLimit
 from pynicotine.slskmessages import SetWaitPort
@@ -336,8 +337,6 @@ class NetworkThread(Thread):
     CONNECTION_MAX_IDLE_GHOST = 10
     CONNECTION_BACKLOG_LENGTH = 65535      # OS limit can be lower
     MAX_INCOMING_MESSAGE_SIZE = 469762048  # 448 MiB, to leave headroom for large shares
-    SOCKET_READ_BUFFER_SIZE = 1048576      # OS limit can be lower
-    SOCKET_WRITE_BUFFER_SIZE = 1048576     # OS limit can be lower
     ALLOWED_PEER_CONN_TYPES = {
         ConnectionType.PEER,
         ConnectionType.FILE,
@@ -346,10 +345,10 @@ class NetworkThread(Thread):
     ERROR_NOT_CONNECTED = OSError(errno.ENOTCONN, strerror(errno.ENOTCONN))
     ERROR_TIMED_OUT = OSError(errno.ETIMEDOUT, strerror(errno.ETIMEDOUT))
 
-    # Looping max ~60 times per second (SLEEP_MIN_IDLE) on high activity
+    # Looping max ~240 times per second (SLEEP_MIN_IDLE) on high activity
     # ~20 (SLEEP_MAX_IDLE + SLEEP_MIN_IDLE) by default
-    SLEEP_MAX_IDLE = 0.0333
-    SLEEP_MIN_IDLE = 0.0166
+    SLEEP_MAX_IDLE = 0.04584
+    SLEEP_MIN_IDLE = 0.00416
 
     try:
         import resource
@@ -402,6 +401,7 @@ class NetworkThread(Thread):
         self._server_timeout_time = None
         self._server_timeout_value = -1
         self._manual_server_disconnect = False
+        self._manual_server_reconnect = False
         self._server_relogged = False
 
         self._parent_conn = None
@@ -460,8 +460,6 @@ class NetworkThread(Thread):
 
         self._listen_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self._listen_socket.setblocking(False)
-        self._listen_socket.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, self.SOCKET_READ_BUFFER_SIZE)
-        self._listen_socket.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, self.SOCKET_WRITE_BUFFER_SIZE)
         self._num_sockets += 1
 
         # On platforms other than Windows, SO_REUSEADDR is necessary to allow binding
@@ -498,7 +496,7 @@ class NetworkThread(Thread):
     def _bind_listen_port(self):
 
         if not self._bind_socket_interface(self._listen_socket):
-            self._set_server_timer()
+            self._set_server_timer(use_fixed_timeout=True)
             log.add(_("Specified network interface '%s' is not available"), self._interface_name)
             return False
 
@@ -509,7 +507,7 @@ class NetworkThread(Thread):
             self._listen_socket.listen(self.CONNECTION_BACKLOG_LENGTH)
 
         except OSError as error:
-            self._set_server_timer()
+            self._set_server_timer(use_fixed_timeout=True)
             log.add(_("Cannot listen on port %(port)s. Ensure no other application uses it, or choose a "
                       "different port. Error: %(error)s"), {"port": self._listen_port, "error": error})
             self._listen_port = None
@@ -1077,9 +1075,12 @@ class NetworkThread(Thread):
 
     # Server Connection #
 
-    def _set_server_timer(self):
+    def _set_server_timer(self, use_fixed_timeout=False):
 
-        if self._server_timeout_value == -1:
+        if use_fixed_timeout:
+            self._server_timeout_value = 5
+
+        elif self._server_timeout_value == -1:
             # Add jitter to spread out connection attempts from Nicotine+ clients
             # in case server goes down
             self._server_timeout_value = random.randint(5, 15)
@@ -1160,6 +1161,7 @@ class NetworkThread(Thread):
         self._portmapper = msg.portmapper
 
         self._manual_server_disconnect = False
+        self._manual_server_reconnect = False
         self._server_timeout_time = None
 
         ip_address, port = msg.addr
@@ -1176,8 +1178,6 @@ class NetworkThread(Thread):
         )
 
         sock.setblocking(False)
-        sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, self.SOCKET_READ_BUFFER_SIZE)
-        sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, self.SOCKET_WRITE_BUFFER_SIZE)
         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 
         # Detect if our connection to the server is still alive
@@ -1497,11 +1497,15 @@ class NetworkThread(Thread):
             self._server_relogged = False
 
         if not self._manual_server_disconnect:
-            self._set_server_timer()
+            self._set_server_timer(use_fixed_timeout=self._manual_server_reconnect)
 
         self._server_address = None
         self._server_username = None
-        events.emit_main_thread("server-disconnect", self._manual_server_disconnect)
+
+        events.emit_main_thread(
+            "server-disconnect",
+            ServerDisconnect(manual_disconnect=self._manual_server_disconnect)
+        )
 
     def _send_message_to_server(self, msg):
         self._process_outgoing_messages([msg])
@@ -1656,8 +1660,6 @@ class NetworkThread(Thread):
             try:
                 incoming_sock, incoming_addr = self._listen_socket.accept()
                 incoming_sock.setblocking(False)
-                incoming_sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, self.SOCKET_READ_BUFFER_SIZE)
-                incoming_sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, self.SOCKET_WRITE_BUFFER_SIZE)
                 incoming_sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 
             except OSError as error:
@@ -1715,8 +1717,6 @@ class NetworkThread(Thread):
         )
 
         sock.setblocking(False)
-        sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, self.SOCKET_READ_BUFFER_SIZE)
-        sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, self.SOCKET_WRITE_BUFFER_SIZE)
         sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
 
         try:
@@ -1883,7 +1883,9 @@ class NetworkThread(Thread):
         file_upload = self._file_upload_msgs[conn]
 
         if file_upload.offset is not None:
-            return None
+            # No more incoming messages on this connection after receiving the
+            # file offset. If peer sends something anyway, clear it.
+            return len(in_buffer)
 
         msg_size = idx = 8
         msg = self._unpack_network_message(
@@ -1920,41 +1922,39 @@ class NetworkThread(Thread):
 
         return idx
 
-    def _write_download_file(self, file_download, in_buffer):
+    def _write_download_file(self, file_download, data, data_len):
 
         try:
-            if not in_buffer:
+            if not data:
                 return
 
-            added_bytes_len = len(in_buffer)
-            file_download.speed += added_bytes_len
-            self._total_download_bandwidth += added_bytes_len
+            file_download.speed += data_len
+            self._total_download_bandwidth += data_len
 
-            file_download.file.write(in_buffer)
-            file_download.leftbytes -= added_bytes_len
+            file_download.file.write(data)
+            file_download.leftbytes -= data_len
 
         finally:
             # Release memoryview in case of critical error
-            in_buffer = None
+            data = None
 
-    def _process_download(self, conn, in_buffer):
+    def _process_download(self, conn, data, data_len):
 
         file_download = self._file_download_msgs[conn]
         idx = file_download.leftbytes
 
         try:
-            if len(in_buffer) > idx:
-                self._write_download_file(file_download, memoryview(in_buffer)[:idx])
+            if data_len > idx:
+                self._write_download_file(file_download, memoryview(data)[:idx], idx)
             else:
-                self._write_download_file(file_download, in_buffer)
+                self._write_download_file(file_download, data, data_len)
 
         except (OSError, ValueError) as error:
             events.emit_main_thread(
                 "download-file-error",
                 username=conn.init.target_user, token=file_download.token, error=error
             )
-            self._close_connection(conn)
-            return None
+            return False  # Close the connection
 
         # Download finished
         if file_download.leftbytes <= 0:
@@ -1963,10 +1963,9 @@ class NetworkThread(Thread):
                 username=conn.init.target_user, token=file_download.token,
                 bytes_left=file_download.leftbytes
             )
-            self._close_connection(conn)
-            return None
+            return False  # Close the connection
 
-        return idx
+        return True
 
     def _process_upload(self, conn, num_sent_bytes, current_time):
 
@@ -1996,8 +1995,7 @@ class NetworkThread(Thread):
                 "upload-file-error",
                 username=conn.init.target_user, token=file_upload.token, error=error
             )
-            self._close_connection(conn)
-            return False
+            return False  # Close the connection
 
         file_upload.speed += num_sent_bytes
         self._total_upload_bandwidth += num_sent_bytes
@@ -2012,7 +2010,7 @@ class NetworkThread(Thread):
         return True
 
     def _process_file_input(self, conn):
-        """Reads file data/messages from the input buffer of a 'F' connection."""
+        """Reads file messages from the input buffer of a 'F' connection."""
 
         in_buffer = conn.in_buffer
         idx = 0
@@ -2020,9 +2018,6 @@ class NetworkThread(Thread):
         if conn not in self._file_init_msgs:
             idx = self._process_file_init_message(conn, in_buffer)
 
-        elif conn in self._file_download_msgs:
-            idx = self._process_download(conn, in_buffer)
-
         elif conn in self._file_upload_msgs:
             idx = self._process_file_offset_message(conn, in_buffer)
 
@@ -2387,6 +2382,10 @@ class NetworkThread(Thread):
             self._manual_server_disconnect = True
             self._close_connection(self._server_conn)
 
+        elif msg_class is ServerReconnect:
+            self._manual_server_reconnect = True
+            self._close_connection(self._server_conn)
+
         elif msg_class is DownloadFile:
             conn = self._conns.get(msg.sock)
 
@@ -2488,12 +2487,14 @@ class NetworkThread(Thread):
             return
 
         try:
-            self._write_data(conn, current_time)
+            if self._write_data(conn, current_time):
+                return
 
         except (OSError, ValueError) as error:
             log.add_conn("Cannot write data to connection %s, closing connection. Error: %s",
                          (conn.addr, error))
-            self._close_connection(conn)
+
+        self._close_connection(conn)
 
     def _process_ready_sockets(self, current_time):
 
@@ -2611,8 +2612,9 @@ class NetworkThread(Thread):
     def _read_data(self, conn, current_time):
 
         sock = conn.sock
-        use_download_limit = (self._download_limit_split and conn in self._file_download_msgs)
         current_recv_size = conn.recv_size
+        is_file_download = (conn in self._file_download_msgs)
+        use_download_limit = (self._download_limit_split and is_file_download)
 
         if use_download_limit:
             download_limit = (self._download_limit_split - self._conns_downloaded[conn])
@@ -2622,7 +2624,17 @@ class NetworkThread(Thread):
 
         data = sock.recv(current_recv_size)
         data_len = len(data)
-        conn.in_buffer += data
+
+        if not data:
+            return False  # Close the connection
+
+        # An intermediate buffer is useless when downloading a file. Write to the
+        # file immediately, and let the OS handle buffering when necessary.
+        if not is_file_download:
+            conn.in_buffer += data
+
+        elif not self._process_download(conn, data, data_len):
+            return False  # Close the connection
 
         if use_download_limit:
             self._conns_downloaded[conn] += data_len
@@ -2635,17 +2647,13 @@ class NetworkThread(Thread):
             conn.recv_size //= 2
 
         conn.last_active = current_time
-
-        if not data:
-            return False
-
         return True
 
     def _write_data(self, conn, current_time):
 
         sock = conn.sock
         out_buffer = conn.out_buffer
-        is_file_upload = conn in self._file_upload_msgs
+        is_file_upload = (conn in self._file_upload_msgs)
 
         if is_file_upload and self._upload_limit_split:
             limit = (self._upload_limit_split - self._conns_uploaded[conn])
@@ -2662,14 +2670,14 @@ class NetworkThread(Thread):
         del out_buffer[:num_bytes_sent]
 
         if is_file_upload and not self._process_upload(conn, num_bytes_sent, current_time):
-            # Connection was closed, stop here
-            return
+            return False  # Close the connection
 
         if not out_buffer:
             # Nothing else to send, stop watching connection for writes
             self._modify_connection_events(conn, selectors.EVENT_READ)
 
         conn.last_active = current_time
+        return True
 
     # Networking Loop #
 
@@ -2700,9 +2708,12 @@ class NetworkThread(Thread):
             if not self._should_process_queue:
                 if self._server_timeout_time and (self._server_timeout_time - current_time) <= 0:
                     self._server_timeout_time = None
-                    events.emit_main_thread("server-reconnect")
+                    events.emit_main_thread(
+                        "server-reconnect",
+                        ServerReconnect(manual_reconnect=self._manual_server_reconnect)
+                    )
 
-                time.sleep(self.SLEEP_MAX_IDLE)
+                time.sleep(self.SLEEP_MAX_IDLE + self.SLEEP_MIN_IDLE)
                 continue
 
             # Process queue messages
