@@ -21,9 +21,9 @@ import shutil
 
 from unittest import TestCase
 
-from pynicotine import slskmessages
 from pynicotine.config import config
 from pynicotine.core import core
+from pynicotine.slskmessages import FileAttribute
 from pynicotine.transfers import TransferStatus
 
 CURRENT_FOLDER_PATH = os.path.dirname(os.path.realpath(__file__))
@@ -39,8 +39,8 @@ class UploadsTest(TestCase):
 
     def setUp(self):
 
-        config.data_folder_path = DATA_FOLDER_PATH
-        config.config_file_path = os.path.join(DATA_FOLDER_PATH, "temp_config")
+        config.set_data_folder(DATA_FOLDER_PATH)
+        config.set_config_file(os.path.join(DATA_FOLDER_PATH, "temp_config"))
 
         if not os.path.exists(DATA_FOLDER_PATH):
             os.makedirs(DATA_FOLDER_PATH)
@@ -89,8 +89,8 @@ class UploadsTest(TestCase):
         self.assertEqual(transfer.size, 27231044)
         self.assertEqual(transfer.current_byte_offset, 27231044)
         self.assertEqual(transfer.file_attributes, {
-            slskmessages.FileAttribute.BITRATE: 792,
-            slskmessages.FileAttribute.DURATION: 268
+            FileAttribute.BITRATE: 792,
+            FileAttribute.DURATION: 268
         })
 
     def test_save_uploads(self):
